"use strict";

const astUtil = require("./util/ast");

const ELEMENTS = {
  FOR: "for"
};
const ATTRIBUTES = {
  KEY_ITEM: "keyItem",
  ITEM: "item",
  OF: "of",
  INDEX: "index"
};

function addMapParam(types, params, attributes, attributeKey) {
  var attribute = attributes[attributeKey];

  if (attribute && attribute.value) {
    params.push(types.Identifier(attribute.value.value));
  } else {
    params.push(types.Identifier(attributeKey));
  }
}

module.exports = function (babel) {
  const types = babel.types;
  return function (path) {
    const node = path.node;
    const attributes = astUtil.getAttributeMap(node);
    const children = astUtil.getChildren(types, node);
    const returnExpression = astUtil.getSanitizedExpressionForContent(types, children);
    const mapParams = [];

    if (!attributes[ATTRIBUTES.OF]) {
      return;
    }

    if (!children.length) {
      return returnExpression;
    }

    if (attributes[ATTRIBUTES.KEY_ITEM]) {
      addMapParam(types, mapParams, attributes, ATTRIBUTES.KEY_ITEM);
      addMapParam(types, mapParams, attributes, ATTRIBUTES.INDEX);
      return types.callExpression(types.memberExpression(types.callExpression(types.memberExpression(types.identifier('Object'), types.identifier('keys')), [astUtil.getExpression(attributes[ATTRIBUTES.OF])]), types.identifier('map')), [types.functionExpression(null, mapParams, types.blockStatement([types.returnStatement(returnExpression)])), types.identifier('this')]);
    } else {
      addMapParam(types, mapParams, attributes, ATTRIBUTES.ITEM);
      addMapParam(types, mapParams, attributes, ATTRIBUTES.INDEX);
      return types.callExpression(types.memberExpression(types.logicalExpression('||', astUtil.getExpression(attributes[ATTRIBUTES.OF]), types.arrayExpression()), types.identifier('map')), [types.functionExpression(null, mapParams, types.blockStatement([types.returnStatement(returnExpression)])), types.identifier('this')]);
    }
  };
};