"use strict";

require("core-js/modules/web.dom.iterable");

const astUtil = require("./util/ast");

const ATTRIBUTES = {
  TEST: 'test'
};

function getBlocks(t, nodes) {
  const result = {
    ifBlock: [],
    elseBlock: []
  };
  nodes.forEach(node => {
    if (node.type == 'JSXElement') {
      if (node.openingElement.name.name == 'else') {
        result.elseBlock.push(...astUtil.getChildren(t, node));
      } else {
        result.ifBlock.push(node);
      }
    }
  });
  return result;
}

module.exports = function (babel) {
  const types = babel.types;
  return function (path) {
    const node = path.node;
    const key = astUtil.getKey(node);
    const attributes = astUtil.getAttributeMap(node);
    const children = astUtil.getChildren(types, node);
    const blocks = getBlocks(types, children);

    if (!attributes[ATTRIBUTES.TEST]) {
      return;
    }

    const ifBlock = astUtil.getSanitizedExpressionForContent(types, blocks.ifBlock, key);
    const elseBlock = astUtil.getSanitizedExpressionForContent(types, blocks.elseBlock, key);
    return types.conditionalExpression(astUtil.getExpression(attributes[ATTRIBUTES.TEST]), ifBlock, elseBlock);
  };
};